@@ -514,6 +514,10 @@ sub mtr_report_stats ($$$$) {
       # if a test case has to be retried it should have the result MTR_RES_FAILED in jUnit XML
       if ($test->{'result'} eq "MTR_RES_FAILED" || $test->{'retries'} > 0) {
         my $logcontents = $test->{'logfile-failed'} || $test->{'logfile'};
+        # remove any double ] that would end the cdata
+        $logcontents =~ s/]]/\x{fffd}/g;
+        # replace wide characters that aren't allowed in XML 1.0
+        $logcontents =~ s/[\x00-\x08\x0B\x0C\x0E-\x1F]/\x{fffd}/g;
 
         $xml_report .= qq(>\n\t\t\t<failure message="" type="MTR_RES_FAILED">\n<![CDATA[$logcontents]]>\n\t\t\t</failure>\n\t\t</testcase>\n);
       } elsif ($test->{'result'} eq "MTR_RES_SKIPPED" && $test->{'disable'}) {
@@ -530,9 +534,9 @@ sub mtr_report_stats ($$$$) {
     # save to file
     my $xml_file = $::opt_xml_report;
 
-    open XML_FILE, ">", $xml_file or die "Cannot create file $xml_file: $!";
-    print XML_FILE $xml_report;
-    close XML_FILE;
+    open (my $XML_UFILE, '>:encoding(UTF-8)', $xml_file) or die 'Cannot create file $xml_file: $!';
+    print $XML_UFILE $xml_report;
+    close $XML_UFILE or warn "File close failed!";
   }
 
   if (@$extra_warnings)
